@@ -36,7 +36,7 @@ namespace KWayland
 namespace Client
 {
 
-class Shell::Private
+class Q_DECL_HIDDEN Shell::Private
 {
 public:
     WaylandPointer<wl_shell, wl_shell_destroy> shell;
@@ -124,7 +124,7 @@ Shell::operator wl_shell*() const
     return d->shell;
 }
 
-class ShellSurface::Private
+class Q_DECL_HIDDEN ShellSurface::Private
 {
 public:
     Private(ShellSurface *q);
@@ -262,9 +262,9 @@ void ShellSurface::Private::pingCallback(void *data, wl_shell_surface *shellSurf
 
 void ShellSurface::Private::popupDoneCallback(void *data, wl_shell_surface *shellSurface)
 {
-    // not needed, we don't have popups
-    Q_UNUSED(data)
-    Q_UNUSED(shellSurface)
+    auto s = reinterpret_cast<ShellSurface::Private*>(data);
+    Q_ASSERT(s->surface == shellSurface);
+    emit s->q->popupDone();
 }
 
 void ShellSurface::setup(wl_shell_surface *surface)
@@ -315,6 +315,18 @@ void ShellSurface::setTransient(Surface *parent, const QPoint &offset, Transient
     wl_shell_surface_set_transient(d->surface, *parent, offset.x(), offset.y(), wlFlags);
 }
 
+void ShellSurface::setTransientPopup(Surface *parent, Seat *grabbedSeat, quint32 grabSerial, const QPoint &offset, TransientFlags flags)
+{
+    Q_ASSERT(isValid());
+    Q_ASSERT(parent);
+    Q_ASSERT(grabbedSeat);
+    uint32_t wlFlags = 0;
+    if (flags.testFlag(TransientFlag::NoFocus)) {
+        wlFlags |= WL_SHELL_SURFACE_TRANSIENT_INACTIVE;
+    }
+    wl_shell_surface_set_popup(d->surface, *grabbedSeat, grabSerial, *parent, offset.x(), offset.y(), wlFlags);
+}
+
 void ShellSurface::requestMove(Seat *seat, quint32 serial)
 {
     Q_ASSERT(isValid());
